package video.jmf;

/*
 * Copyright (c) 1996-2001 Sun Microsystems, Inc. All Rights Reserved.
 *
 * Sun grants you ("Licensee") a non-exclusive, royalty free, license to use,
 * modify and redistribute this software in source and binary code form,
 * provided that i) this copyright notice and license appear on all copies of
 * the software; and ii) Licensee does not utilize the software in a manner
 * which is disparaging to Sun.
 *
 * This software is provided "AS IS," without a warranty of any kind. ALL
 * EXPRESS OR IMPLIED CONDITIONS, REPRESENTATIONS AND WARRANTIES, INCLUDING ANY
 * IMPLIED WARRANTY OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE OR
 * NON-INFRINGEMENT, ARE HEREBY EXCLUDED. SUN AND ITS LICENSORS SHALL NOT BE
 * LIABLE FOR ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING
 * OR DISTRIBUTING THE SOFTWARE OR ITS DERIVATIVES. IN NO EVENT WILL SUN OR ITS
 * LICENSORS BE LIABLE FOR ANY LOST REVENUE, PROFIT OR DATA, OR FOR DIRECT,
 * INDIRECT, SPECIAL, CONSEQUENTIAL, INCIDENTAL OR PUNITIVE DAMAGES, HOWEVER
 * CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY, ARISING OUT OF THE USE OF
 * OR INABILITY TO USE SOFTWARE, EVEN IF SUN HAS BEEN ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGES.
 *
 * This software is not designed or intended for use in on-line control of
 * aircraft, air traffic, aircraft navigation or aircraft communications; or in
 * the design, construction, operation or maintenance of any nuclear
 * facility. Licensee represents and warrants that it will not use or
 * redistribute the Software for such purposes.
 */


import com.sun.media.controls.VFlowLayout;

import javax.media.*;
import javax.media.control.MonitorControl;
import javax.media.datasink.DataSinkListener;
import javax.media.format.VideoFormat;
import javax.media.protocol.DataSource;
import javax.media.protocol.FileTypeDescriptor;
import java.awt.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.io.IOException;
import java.util.StringTokenizer;

public class RunVideoPanel extends Frame implements ItemListener, ActionListener {

    // GUI components
    
    Panel jPanel1 = new Panel();
    Choice comboFileType = new Choice();
    VFlowLayout verticalFlowLayout1 = new VFlowLayout();
    Label jLabel1 = new Label();
    Checkbox checkVideo = new Checkbox();
    Choice comboEncoding = new Choice();
    Choice comboSize = new Choice();


    Panel jPanel2 = new Panel();
    Panel jPanel3 = new Panel();

    Button buttonStart = new Button();
    Button buttonExit = new Button();
    Button buttonEnd = new Button();
    Panel panel1 = new Panel();
    GridLayout gridLayout1 = new GridLayout();
    GridLayout gridLayout2 = new GridLayout();

    // JMF objects
    Processor processor = null;
    DataSink datasink = null;
    Component monitor = null;
    DataSource datasource = null;
    String outputType = "video.quicktime";

    public RunVideoPanel() {
        createGUI();
        fillGUI();
    }

    public DataSink getDataSink() {
        DataSink dataSink = new DataSink() {
            public void setOutputLocator(MediaLocator mediaLocator) {
            }

            public MediaLocator getOutputLocator() {
                return null;
            }

            public Object[] getControls() {
                return null;
            };

            public Object getControl(String s) {
                return null;
            };

            public void setSource(DataSource ds) {

            }

            public void start() throws IOException {
            }

            public void stop() throws IOException {
            }

            public void open() throws IOException, SecurityException {
            }

            public void close() {
            }

            public String getContentType() {
                return "";
            }

            public void addDataSinkListener(DataSinkListener dataSinkListener) {
            }

            public void removeDataSinkListener(DataSinkListener dataSinkListener) {
            }
        };
        return dataSink;
    }

    private void startMonitoring() {
        // Close the previous processor, which in turn closes the capture device
        if (processor != null) {
            processor.stop();
            processor.close();
        }
        // Remove the previous monitor
        if (monitor != null) {
            panel1.remove(monitor);
            monitor = null;
        }


        VideoFormat vf = null;


        if (checkVideo.getState()) {
            String encoding = comboEncoding.getSelectedItem();
            String strSize = comboSize.getSelectedItem();
            StringTokenizer st = new StringTokenizer(strSize, "x");
            int sizeX = Integer.parseInt(st.nextToken());
            int sizeY = Integer.parseInt(st.nextToken());
            Dimension size = new Dimension(sizeX, sizeY);

            vf = new VideoFormat(encoding, size, Format.NOT_SPECIFIED,
                    null, 15f);
        }

        // Use video.jmf.CaptureUtil to create a monitored capture datasource
        datasource = CaptureUtil.getCaptureDS(vf);

        if (datasource != null) {
            // Set the preferred content type for the Processor's output
            outputType = "video.quicktime";
            if (comboFileType.getSelectedItem().equals("AVI"))
                outputType = "video.x_msvideo";
            FileTypeDescriptor ftd = new FileTypeDescriptor(outputType);
            Format[] formats = null;

            if (vf != null) {
                formats = new Format[]{new VideoFormat(null)};
            }

            formats = new Format[]{new VideoFormat(null)};

            ProcessorModel pm = new ProcessorModel(datasource, formats, ftd);
            try {
                processor = Manager.createRealizedProcessor(pm);
            } catch (Exception me) {
                System.err.println(me);
                // Make sure the capture devices are released
                datasource.disconnect();
                return;
            }

            // Get the monitor control:
            // Since there are more than one MonitorControl objects
            // exported by the DataSource, we get the specific one
            // that is also the video.jmf.MonitorStream object.
            MonitorControl mc = (MonitorControl) datasource.getControl("jmfsample.video.jmf.MonitorStream");
            if (mc != null) {
                monitor = mc.getControlComponent();
                panel1.add(monitor);
                // Make sure the monitor is enabled
                mc.setEnabled(true);
                pack();
            }
        }
    }

    private void startCapture() {
        enableComponents(false);
        buttonStart.setLabel("Pause");
        buttonEnd.setEnabled(true);

        // Get the processor's output, create a DataSink and connect the two.
        DataSource outputDS = processor.getDataOutput();
        try {
            MediaLocator ml = new MediaLocator("file:capture." +
                    (outputType.equals("video.x_msvideo") ? "avi" : "mov"));
            datasink = Manager.createDataSink(outputDS, ml);
            datasink.open();
            datasink.start();
        } catch (Exception e) {
            System.err.println(e);
        }
        processor.start();
        System.out.println("Started saving...");
    }

    private void pauseCapture() {
        processor.stop();
        buttonStart.setLabel("Resume");
    }

    private void resumeCapture() {
        processor.start();
        buttonStart.setLabel("Pause");
    }

    private void stopCapture() {
        // Stop the capture and the file writer (DataSink)
        processor.stop();
        processor.close();
        datasink.close();
        processor = null;
        buttonEnd.setEnabled(false);
        // Restart monitoring
        startMonitoring();
        buttonStart.setLabel("Start");
        enableComponents(true);
        System.out.println("Done saving.");
    }


    private void enableComponents(boolean state) {
        comboFileType.setEnabled(state);
        comboEncoding.setEnabled(state);
        comboSize.setEnabled(state);

        buttonExit.setEnabled(state);
    }


    void exit() {
        if (processor != null)
            processor.close();
        System.exit(0);
    }

    private void fillGUI() {
        comboFileType.add("QuickTime");
        comboFileType.add("AVI");

        comboEncoding.add("RGB"); // Add YUV, etc.

        // Assume these sizes are available - might need modification
        // for certain capture cards / cameras.
        comboSize.add("160x120");
        comboSize.add("320x240");
        comboSize.add("640x480"); // Add more sizes if your card supports
        // Or detect them by getting a format list from the capture device info.



        // Initialize listeners

        comboSize.addItemListener(this);
        comboEncoding.addItemListener(this);

        comboFileType.addItemListener(this);

        buttonStart.addActionListener(this);
        buttonEnd.addActionListener(this);

        // Create the datasource and processor
        startMonitoring();
    }

    public void itemStateChanged(ItemEvent sce) {
        Object source = sce.getSource();
        if (source == comboSize || source == comboEncoding ||
                source == comboFileType) {
            startMonitoring();
        }

    }

    public void actionPerformed(ActionEvent ae) {
        String action = ae.getActionCommand();
        if (action.equals("Start")) {
            startCapture();
        } else if (action.equals("Pause")) {
            pauseCapture();
        } else if (action.equals("Resume")) {
            resumeCapture();
        } else if (action.equals("End")) {
            stopCapture();
        } else if (action.equals("Exit")) {
            exit();
        }
    }

    private void createGUI() {
        this.setTitle("video.jmf.JVidCap");
        jPanel1.setLayout(verticalFlowLayout1);
        jLabel1.setText("File Format");
        checkVideo.setState(true);
        checkVideo.setEnabled(false);
        checkVideo.setLabel("Video");


        jPanel2.setLayout(gridLayout2);

        jPanel3.setLayout(gridLayout1);
        buttonStart.setLabel("Start");
        buttonExit.setLabel("Exit");
        buttonExit.addActionListener(this);
        buttonEnd.setEnabled(false);
        buttonEnd.setLabel("End");
        this.add(jPanel1, BorderLayout.WEST);
        jPanel1.add(jLabel1);
        jPanel1.add(comboFileType);
        jPanel1.add(checkVideo);
        jPanel1.add(comboEncoding);
        jPanel1.add(comboSize);

        jPanel1.add(jPanel3);

        jPanel1.add(jPanel2);

        jPanel1.add(buttonStart);
        jPanel1.add(buttonEnd);
        jPanel1.add(buttonExit);
        this.add(panel1, BorderLayout.CENTER);

    }

    public static void main(String[] args) {
        RunVideoPanel JVidCap1 = new RunVideoPanel();
        JVidCap1.setVisible(true);
        JVidCap1.pack();
    }

}
